package com.topcoder.persistence.relation;

/**
 * Interface that abstracts any operation that takes two relations as input
 * and fills with tuples a result relation.
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * @author TCSDESIGNER
 */
public interface BinaryOperation {

	/**
	 * Computes the result relation from the two relations.
	 * 
	 * @param leftRelation the left relation
	 * @param rightRelation the right relation
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if any argument is null or the result relation is not empty and typeless
	 * @throws IncompatibleRelationsException if the implementation decides the relations are not compatible
	 * @throws OperationException if any implementation specific error occurs
	 */
	public void compute(Relation leftRelation, Relation rightRelation, Relation resultRelation) throws IllegalArgumentException, IncompatibleRelationsException, OperationException;

}
