package com.topcoder.persistence.relation;

import java.util.HashMap;
import java.util.Map;

import com.topcoder.util.config.ConfigManagerException;

/**
 * The relation manager class provides relation management services (by name).
 * However, the main functionality is the ability to obtain relations from
 * configuration files.
 * 
 * The class reads the relation definition information in the constructor.
 * The relation can be easily retrieved afterwards using the getters.
 * 
 * Note:
 * 
 * The relations can be used just as well without having to add them to the
 * manager. The manager is only meant to provide easy access to relations
 * by name, if such functionality is needed.
 * 
 * Thread safety:
 * 
 * Not thread safe because of the tuples map that is changed by several methods.
 * It can be made thread safe by synchronizing all methods.
 * 
 * @author TCSDESIGNER
 */
public class RelationManager {
	
	/**
	 * Constant for the default namespace to be used in the constructor with no arguments.
	 * Public so that if some other component needs to specify the namespace used by this
	 * component, it doesn't have to hardcode it.
	 */
	public static final String DEFAULT_NAMESPACE = "com.topcoder.persistence.relation";
	
	/**
	 * The relation name to relation mapping.
	 * Maps strings to Relation instances.
	 * Cannot be null or have null keys and values.
	 * Initialized with HashMap.
	 * Filled initially in the constructor but used or mutated by all the other methods.
	 */
	private Map relations = new HashMap();
	
	/**
	 * Constructor.
	 * Reads the relation configuration from the default namespace. 
	 * 
	 * Implementation notes:
	 * 
	 * Calls the other constructor passing DEFAULT_NAMESPACE as parameter.
	 * 
	 * @throws ConfigManagerException a configuration related exception coming from
	 * the Configuration Manager component but it is also thrown directly in case
	 * a required property missing or wraps a reflection exception 
	 */
	public RelationManager() throws ConfigManagerException {
	}

	/**
	 * Constructor.
	 * Reads the relation configuration from the given namespace. 
	 * 
	 * Implementation notes:
	 * 
	 * 1. For each property from the namespace, do steps 2-13
	 * 2. Read attributes subproperty.
	 * 3. Read each subproperty of attributes (attribute names, types)
	 * 4. Build a tuple type from the attribute info
	 * 5. Create relation
	 * 6. Read tuple_constraints subproperty
	 * 7. Read each subproperty of tuple_constraints
	 * 8. Create the constraint using reflection (passing the constraint property as param)
	 * 9. Read relation_constraints subproperty
	 * 10. Read each subproperty of relation_constraints
	 * 11. Create the constraint using reflection (passing the constraint property as param)
	 * 12. Add constraints to relation
	 * 13. Put name-relation in map
	 * 
	 * @param namespace the namespace to load from
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws ConfigManagerException a configuration related exception coming from
	 * the Configuration Manager component but it is also thrown directly in case
	 * a required property missing or wraps a reflection exception 
	 */
	public RelationManager(String namespace) throws IllegalArgumentException, ConfigManagerException {
	}
	
	/**
	 * Adds a relation to the manager.
	 * If a relation with the same name exists, nothing is done.
	 * 
	 * Implementation details:
	 * 
	 * - checks the map for name
	 * - adds relation to the map
	 * 
	 * @param relation the relation to add
	 * 
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean addRelation(Relation relation) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Removes a relation from the manager (by name).
	 * If a relation with that name doesn't exists, nothing is done.
	 * 
	 * Implementation details:
	 * 
	 * - removes from the map
	 * 
	 * @param name the name of the relation to remove
	 * 
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public boolean removeRelation(String name) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears the relations.
	 * 
	 * Implementation details:
	 *
	 * Clears the map.
	 */
	public void clearRelations() {
	}
	
	/**
	 * Returns the names of the defined relations.
	 * 
	 * Implementation details:
	 *
	 * Returns the key set of the map.
	 * 
	 * @return the names of the defined relations
	 */
	public String[] getRelationNames() {
		return null;
	}
	
	/**
	 * Gets a relation from the manager (by name).
	 * If a relation with that name doesn't exists, null is returned
	 * 
	 * Implementation details:
	 * 
	 * - gets from the map
	 * 
	 * @param name the name of the relation to get
	 * 
	 * @return the relation or null if no relation with the given name
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Relation getRelation(String name) throws IllegalArgumentException {
		return null;
	}

}
