package com.topcoder.persistence.relation.constraints;

import com.topcoder.util.collection.typesafeenum.Enum;

/**
 * Enumeration that specifies the comparison types that can be
 * made between attributes and values.
 * 
 * This enumeration has constants for the usual operators
 * (==, <, >, <=, =>).
 * 
 * Implementation notes:
 * 
 * Extends the Enum class from Typesafe Enum component.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no problems. 
 */
public class ComparisonType extends Enum {

	/**
	 * Constant (enumeration value) for the equal (==) comparison type.
	 */
	public static final ComparisonType EQUAL = new ComparisonType();

	/**
	 * Constant (enumeration value) for the greater than (>) comparison type.
	 */
	public static final ComparisonType GREATER = new ComparisonType();
	
	/**
	 * Constant (enumeration value) for the less than (<) comparison type.
	 */
	public static final ComparisonType LESS = new ComparisonType();
	
	/**
	 * Constant (enumeration value) for the greater than or equal (>=) comparison type.
	 */
	public static final ComparisonType GREATER_EQUAL = new ComparisonType();

	/**
	 * Constant (enumeration value) for the less than or equal (<=) comparison type.
	 */
	public static final ComparisonType LESS_EQUAL = new ComparisonType();
	
	/**
	 * Private constructor that prevents direct instantiation.
	 * 
	 * Implementation notes:
	 *
	 * Does nothing.
	 */
	private ComparisonType() {
	}
	
}
