package com.topcoder.persistence.relation.operations;

import com.topcoder.persistence.relation.OperationException;
import com.topcoder.persistence.relation.Relation;
import com.topcoder.persistence.relation.TupleConstraint;
import com.topcoder.persistence.relation.UnaryOperation;

/**
 * Implementation of a unary operation for the restriction of a relation
 * for a given constraint. The result of this operation is a new 
 * relation that selects only the tuples that meet the given constraint.
 * 
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * Thread safety:
 * 
 * This class is immutable so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public class RestrictOperation implements UnaryOperation {
	
	/**
	 * The constraint that specifies which tuples the restriction operation should keep.
	 * Cannot be null.
	 * Set in the constructor and used in compute.
	 */
	private TupleConstraint constraint = null;
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Sets the constraint attribute.
	 * 
	 * @param constraint the constraint that specifies which tuples the restriction operation should keep
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public RestrictOperation(TupleConstraint constraint) throws IllegalArgumentException {
	}
	
	/**
	 * Returns the restrict operation constraint.
	 * 
	 * Implementation details:
	 * 
	 * Returns the constraint attribute.
	 * 
	 * @return the restrict operation constraint
	 */
	public TupleConstraint getConstraint() {
		return null;
	}

	/**
	 * Computes the restriction of a source relation.
	 * The constraint that selects the tuples to keep is given in the constructor. 
	 * The result relation will contain the tuples from the source relation
	 * that respect the constraint. 
	 * 
	 * Implementation details:
	 * 
	 * The restriction algorithm is simply iterating all tuples from the source
	 * relation and adding to the result relation those that meet the constraint.
	 * Assuming that the external constraint checking for one tuple is done in O(1),
	 * the total complexity is O(n) where n is the size of the source relation.
	 * 
	 * - copy the tuple type from the source relation to the result relation
	 * - calls sourceRelation.getTuplesIterator()
	 * - iterates all tuples using the iterator
	 * - checks if the constraint is true for the tuple
	 * - if true, the tuple is added to the result relation
	 * 
	 * @param sourceRelation the source relation
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if any argument is null or the result relation is not empty and typeless
	 * @throws OperationException wraps any exception thrown by the constraint evaluation   
	 */
	public void compute(Relation sourceRelation, Relation resultRelation) throws IllegalArgumentException, OperationException {
	}

}
